@@ -13,7 +13,7 @@ use Masterkey::Admin::Config;
 use Masterkey::Admin::Torus;
 
 our @ISA = qw();
-our $VERSION = '4.2';
+our $VERSION = '4.3';
 
 sub new {
     my $class = shift();
@@ -94,7 +94,7 @@ sub fieldsWithDefaultValues {
 
 sub new_with_session {
     my $class = shift();
-    my($tabName, $r, $nouser) = @_;
+    my($tabName, $r, $nouser, $identityId, $force_uid) = @_;
 
     my $admin;
     eval {
@@ -106,7 +106,11 @@ sub new_with_session {
     my $domain = $admin->conf("authDomain");
     my $session = undef;
     my $cookies = Apache2::Cookie->fetch($r);
-    my $cookie = $cookies->{"session-$domain"};
+    my $cname = "session-$domain";
+    if ($ENV{MAnewurls}) {
+	$cname .= "-" . $identityId;
+    }
+    my $cookie = $cookies->{$cname};
     if (defined $cookie) {
 	my $cval = $cookie->value();
 	$session = Masterkey::Admin::Session->find($cval);
@@ -116,30 +120,31 @@ sub new_with_session {
     if (!defined $session) {
 	$session = Masterkey::Admin::Session->new();
 	my $cookie = new Apache2::Cookie($r, -path => "/",
-					 -name => "session-$domain",
+					 -name => $cname,
 					 -value => $session->cookie());
 	$cookie->bake($r);
     }
 
+    if (defined $force_uid) {
+	warn "forcing new session's ID to $force_uid";
+	$session->update(user_id => $force_uid);
+    }
+
     my $user = undef;
     goto DONE if $nouser;
 
     my $uid = $session->user_id();
 
-    # On http://ohiolink-test.indexdata.com/ this fails randomly
-    # about half the time, saying "Undefined subroutine
-    # &Apache2::Request called at
-    # /usr/lib/perl5/site_perl/5.8.8/Masterkey/Admin.pm line 115".
-    # We don't understand why, so for now we just comment it out.
-#    if (!$uid) {
-#	# Attempt an immediate login with supplied parameters, if any
-#	my $usefulR = new Apache2::Request($r);
-#	my $user = $usefulR->param("authUser");
-#	my $password = $usefulR->param("authPassword");
-#	if (defined $user && defined $password) {
-#	    warn "### we should log in as $user/$password";
-#	}
-#    }
+    if (!$uid) {
+	use Apache2::Request;
+	# Attempt an immediate login with supplied parameters, if any
+	my $usefulR = Apache2::Request->new($r);
+	my $user = $usefulR->param("authUser");
+	my $password = $usefulR->param("authPassword");
+	if (defined $user && defined $password) {
+	    warn "### we should log in as $user/$password";
+	}
+    }
 
     if (!$uid) {
 	# Check whether we were logged into the Harvester Console,
