@@ -113,16 +113,10 @@ sub _with_dbfile {
     my $dbfile = "/tmp/mkadmin-sessions-$tag.sdbm";
 
     my %h;
-    eval {
-	tie(%h, "SDBM_File", $dbfile, $flags, 0666)
-	    or die "Couldn't write SDBM file '$dbfile': $!";
-	&$callback(\%h);
-    }; if ($@ && !$readWrite) {
-	# Doesn't matter that we're not really tied, since we're not making updates
-	&$callback({});
-    } elsif ($@) {
-	die $@;
-    }
+    tie(%h, "SDBM_File", $dbfile, $flags, 0666)
+	or die "Couldn't write SDBM file '$dbfile': $!";
+
+    &$callback(\%h);
 
     untie %h;
     
